﻿using System;
using System.Collections.Generic;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Security.Cryptography.Xml;
using System.Text;
using System.Threading.Tasks;

namespace KPGBase.Excercises
{
    internal class Excercise04 : BasicExcercise
    {

        public Excercise04(Bitmap canvas) : base(canvas) { }

        /// <summary>
        /// Fill the plane with 12-gonal tiles.
        /// </summary>
        public void SimpleTiling()
        {
            ClearImage();
            Matrix startMatrix = g.Transform.Clone();

            // HERE INSERT YOUR CODE

            Brush trigB = new SolidBrush(Color.GreenYellow);
            g.FillRectangle(trigB, new Rectangle(0, 0, width, height));

            // parameters
            int n = 12;
            int radius = 120; // radius of n-gon, distance from center to one point
            double alpha = (360.0 / n) / 2; // angle between two points and center
            int v = (int)(Math.Cos(alpha * Math.PI / 180) * radius / 2.0); // distance from center to one side

            // reference n-gon to compute needed shift
            Point[] pts = CreateNgon(0, 0, n, radius / 2);
            int shift = (int)Math.Abs(pts[9].Y - pts[10].Y); // distance of points at the top of the n-gon, needed for shift

            // used brushes
            Brush hexB = new SolidBrush(Color.DarkRed);
            Pen line = new Pen(Color.Black, 2);

            int hexLocY = 0;
            int row = 1;

            // save trasform matrix
            Matrix mat = g.Transform.Clone();

            // rows
            while (hexLocY < (height + v))
            {
                // every other row has to be shifted
                int hexLocX = v;
                if (row % 2 == 0)
                    hexLocX = 0;

                // translate to the starting point
                g.Transform = mat;
                g.TranslateTransform(hexLocX, hexLocY);

                // columns
                while (hexLocX < (width + v))
                {
                    g.FillPolygon(hexB, pts);
                    g.DrawPolygon(line, pts);
                    hexLocX += 2 * v;
                    
                    // translate by width of ngon
                    g.TranslateTransform(2*v, 0);
                }
                hexLocY += 2 * v - shift;
                row++;
            }

            // set transform back to original
            g.Transform = startMatrix;
        }

        /// <summary>
        /// Creates an n-gon at location [x, y]
        /// </summary>
        /// <param name="x"> X coordinate </param>
        /// <param name="y"> Y coordinate </param>
        /// <param name="n"> Number of points </param>
        /// <param name="r"> Radius </param>
        /// <returns> Points of the generated n-gon </returns>
        Point[] CreateNgon(int x, int y, int n, int r)
        {
            Point[] points = new Point[n];
            double step = 360 / n * Math.PI / 180f;
            for (int i = 0; i < n; i++)
            {
                points[i] = new Point((int)(x + r * (float)Math.Cos(i * 360 / n * Math.PI / 180f + step / 2)),
                                       (int)(y + r * (float)Math.Sin(i * 360 / n * Math.PI / 180f + step / 2)));
            }
            return points;
        }

        public void TriangleTiling()
        {
            ClearImage();
            Matrix startMatrix = g.Transform.Clone();

            // translate to center
            g.TranslateTransform(width / 2, height / 2);

            // parameters
            int n = 8; // number of triangles in center
            double angle = 360.0 / n; // center angle of one triangle
            double tSide = 40; // triangle side length
            double tHeight = Math.Cos((angle / 2.0) * Math.PI / 180) * tSide; // height of the triangle
            double tBase = Math.Sin((angle / 2.0) * Math.PI / 180) * tSide; // base of the triangle
            
            Pen p = new Pen(Color.White, 2);
            for (int i = 0; i < n; i++)
            {
                // draw first triangle - the one in the center
                Point p0 = new Point(0, 0);
                Point p1 = new Point((int)(p0.X + tHeight), (int)(p0.Y - tBase));
                Point p2 = new Point((int)(p0.X + tHeight), (int)(p0.Y + tBase));
                Point[] pts = new Point[] { p0, p1, p2 };
                g.DrawPolygon(p, pts);

                // draw first "row" of reflected triangles
                List<Point[]> firstRow = DrawReflectedTriangles(pts, angle, tSide, tHeight);

                // draw second "row" of reflected triangles
                List<Point[]> secondRow1 = DrawReflectedTriangles(firstRow[0], angle, tSide, tHeight);
                List<Point[]> secondRow2 = DrawReflectedTriangles(firstRow[1], angle, tSide, tHeight, true);

                // draw third "row"
                List<Point[]> thirdRow1 = DrawReflectedTriangles(secondRow1[0], angle, tSide, tHeight);
                List<Point[]> thirdRow2 = DrawReflectedTriangles(secondRow1[1], angle, tSide, tHeight, true);
                List<Point[]> thirdRow3 = DrawReflectedTriangles(secondRow2[1], angle, tSide, tHeight, true);

                g.RotateTransform((int)angle);
            }

            g.Transform = startMatrix;
        }

        private List<Point[]> DrawReflectedTriangles(Point[] pts, double angle, double tSide, double tHeight, bool bottomOnly = false)
        {
            Pen p = new Pen(Color.White, 2);

            // reflected
            Point[] ptsRef = GetReflectedTriangle(pts, tHeight);
            Point[] ptsTop = GetTopTriangle(pts, tSide, angle);
            Point[] ptsBot = GetBottomTriangle(pts, tSide, angle);

            g.DrawPolygon(p, ptsRef);
            g.DrawPolygon(p, ptsBot);
            if (!bottomOnly)
                g.DrawPolygon(p, ptsTop);

            return new List<Point[]> { ptsTop, ptsBot };
        }

        private Point[] GetBottomTriangle(Point[] pts, double tSide, double angle)
        {
            // distance from the top of the triangle being reflected
            double side2 = 2 * tSide;
            double height2 = Math.Cos((angle / 2.0) * Math.PI / 180) * side2;
            double base2 = Math.Sin((angle / 2.0) * Math.PI / 180) * side2;

            Point p3 = new Point((int)(pts[0].X + height2), pts[0].Y);
            Point p5 = new Point((int)(pts[0].X + height2), (int)(pts[0].Y + base2));

            Point[] ptsBot = new Point[] { pts[2], p3, p5 };
            return ptsBot;
        }

        private Point[] GetTopTriangle(Point[] pts, double tSide, double angle)
        {
            // distance from the top of the triangle being reflected
            double side2 = 2 * tSide;
            double height2 = Math.Cos((angle / 2.0) * Math.PI / 180) * side2;
            double base2 = Math.Sin((angle / 2.0) * Math.PI / 180) * side2;

            Point p3 = new Point((int)(pts[0].X + height2), pts[0].Y);
            Point p4 = new Point((int)(pts[0].X + height2), (int)(pts[0].Y - base2));

            Point[] ptsTop = new Point[] { pts[1], p4, p3 };
            return ptsTop;
        }

        private Point[] GetReflectedTriangle(Point[] pts, double v)
        {
            Point p3 = new Point((int)(pts[0].X + 2 * v), pts[0].Y);
            Point[] ptsRef = new Point[] { p3, pts[1], pts[2]};
            return ptsRef;
        }


        public void SpiralTiling()
        {
            ClearImage();
            Matrix startMatrix = g.Transform.Clone();

            // translate to center
            g.TranslateTransform(width / 2, height / 2);

            // parameters
            int n = 8; // number of triangles in center
            double angle = 360.0 / n; // center angle of one triangle
            double tSide = 40; // triangle side length
            double tHeight = Math.Cos((angle / 2.0) * Math.PI / 180) * tSide; // height of the triangle
            double tBase = Math.Sin((angle / 2.0) * Math.PI / 180) * tSide; // base of the triangle

            Pen p = new Pen(Color.White, 2);
            for (int i = 0; i < n; i++)
            {
                // draw first triangle - the one in the center
                Point p0 = new Point(0, 0);
                Point p1 = new Point((int)(p0.X + tHeight), (int)(p0.Y - tBase));
                Point p2 = new Point((int)(p0.X + tHeight), (int)(p0.Y + tBase));
                Point[] pts = new Point[] { p0, p1, p2 };
                g.DrawPolygon(p, pts);

                // draw first "row" of reflected triangles
                List<Point[]> firstRow = DrawReflectedTriangles(pts, angle, tSide, tHeight);

                // draw second "row" of reflected triangles
                List<Point[]> secondRow1 = DrawReflectedTriangles(firstRow[0], angle, tSide, tHeight);
                List<Point[]> secondRow2 = DrawReflectedTriangles(firstRow[1], angle, tSide, tHeight, true);

                // draw third "row"
                List<Point[]> thirdRow1 = DrawReflectedTriangles(secondRow1[0], angle, tSide, tHeight);
                List<Point[]> thirdRow2 = DrawReflectedTriangles(secondRow1[1], angle, tSide, tHeight, true);
                List<Point[]> thirdRow3 = DrawReflectedTriangles(secondRow2[1], angle, tSide, tHeight, true);

                g.RotateTransform((int)angle);

                // cut in half and shift
                if (i == 3)
                    g.TranslateTransform(p2.X, -p2.Y);
            }

            g.Transform = startMatrix;
        }

    }
}
